package com.aspose.cells.demos.chart;

import javax.servlet.http.HttpServletRequest;

import com.aspose.cells.Axis;
import com.aspose.cells.Chart;
import com.aspose.cells.ChartCollection;
import com.aspose.cells.ChartType;
import com.aspose.cells.Color;
import com.aspose.cells.FileFormatType;
import com.aspose.cells.Font;
import com.aspose.cells.SeriesCollection;
import com.aspose.cells.TickLabelPositionType;
import com.aspose.cells.TickMarkType;
import com.aspose.cells.Title;
import com.aspose.cells.Workbook;
import com.aspose.cells.Worksheet;
import com.aspose.cells.WorksheetCollection;
import com.aspose.cells.demos.DemoBaseServlet;

public class CylinderColumnServlet extends DemoBaseServlet
{
    private static final String TEMPLATE_FILE_PATH_PART = "/WEB-INF/Designer/Chart/Template2.xls";

    protected Workbook createReport(HttpServletRequest request) throws Exception
    {
        int fileFormatType = FileFormatType.EXCEL_97_TO_2003;
        resultFile = "CylinderColumn.xls";

        Workbook wb = new Workbook(getServletContext().getRealPath(TEMPLATE_FILE_PATH_PART));
        int chartType = ChartType.CYLINDER;
        switch (Integer.parseInt(request.getParameter("ChartTypeList")))
        {
            case 1:
            {
                chartType = ChartType.CYLINDER_STACKED;
                break;
            }
        }
        createStaticReport(wb, chartType);
        return wb;
    }

    private void createStaticReport(Workbook workbook, int chartType) throws Exception
    {
        WorksheetCollection worksheets = workbook.getWorksheets();
        Worksheet worksheet = worksheets.get(0);
        //Set the name of worksheet
        worksheet.setName("Cylinder Column");

        //Create chart
        ChartCollection charts = worksheet.getCharts();
        Chart chart = charts.get(charts.add(chartType, 1, 3, 25, 12));

        //Set properties of chart
        chart.setElevation(15);
        chart.setRotationAngle(20);
        chart.setGapWidth(10);
        chart.setDepthPercent(280);
        chart.setShowLegend(false);

        //Set properties of nseries
        SeriesCollection nSeries = chart.getNSeries();
        nSeries.add("B2:C12", true);
        nSeries.setCategoryData("A2:A12");

        //Set properties of chart title
        Title title = chart.getTitle();
        title.setText("Number of Employees");
        Font font1 = title.getTextFont();
        font1.setBold(true);
        font1.setColor(Color.getBlack());
        font1.setSize(12);

        //Set properties of categoryaxis title
        Axis categoryAxis = chart.getCategoryAxis();
        Font font2 = categoryAxis.getTitle().getTextFont();
        font2.setColor(Color.getBlack());
        font2.setName("Arial");
        font2.setSize(10);
        categoryAxis.setMajorTickMark(TickMarkType.OUTSIDE);
        categoryAxis.setMinorTickMark(TickMarkType.NONE);
        categoryAxis.setTickLabelPosition(TickLabelPositionType.LOW);
        categoryAxis.setTickLabelSpacing(2);
        categoryAxis.setTickMarkSpacing(1);

        //Set properties of valueaxis
        Axis valueAxis = chart.getValueAxis();
        Font font3 = valueAxis.getTickLabels().getFont();
        font3.setColor(Color.getBlack());
        font3.setName("Arial");
        font3.setSize(10);
        valueAxis.setMajorTickMark(TickMarkType.OUTSIDE);
        valueAxis.setMinorTickMark(TickMarkType.NONE);
        valueAxis.setTickLabelPosition(TickLabelPositionType.NEXT_TO_AXIS);
    }

}
