package com.aspose.cells.demos.chart;

import javax.servlet.http.HttpServletRequest;

import com.aspose.cells.Axis;
import com.aspose.cells.Cells;
import com.aspose.cells.Chart;
import com.aspose.cells.ChartCollection;
import com.aspose.cells.ChartType;
import com.aspose.cells.Color;
import com.aspose.cells.FileFormatType;
import com.aspose.cells.Font;
import com.aspose.cells.Legend;
import com.aspose.cells.LegendPositionType;
import com.aspose.cells.SeriesCollection;
import com.aspose.cells.TickMarkType;
import com.aspose.cells.Title;
import com.aspose.cells.Workbook;
import com.aspose.cells.Worksheet;
import com.aspose.cells.WorksheetCollection;
import com.aspose.cells.demos.DemoBaseServlet;

public class Line3DServlet extends DemoBaseServlet
{
    private static final String TEMPLATE_FILE_PATH_PART = "/WEB-INF/Designer/Chart/AreaTemplate.xls";

    protected Workbook createReport(HttpServletRequest request) throws Exception
    {
        int fileFormatType = FileFormatType.EXCEL_97_TO_2003;
        resultFile = "Line3D.xls";

        Workbook wb = new Workbook(getServletContext().getRealPath(TEMPLATE_FILE_PATH_PART));
        int cLabelsRotation = Integer.parseInt(request.getParameter("CLabelsRotation"));
        int vLabelsRotation = Integer.parseInt(request.getParameter("VLabelsRotation"));
        int majorTickMarkType = Integer.parseInt(request.getParameter("MajorTickMarkType"));
        int minorTickMarkType = Integer.parseInt(request.getParameter("MinorTickMarkType"));
        createStaticReport(wb, cLabelsRotation, vLabelsRotation, majorTickMarkType,
                minorTickMarkType);
        return wb;
    }

    private void createStaticReport(Workbook workbook, int cLabelsRotation, int vLabelsRotation,
            int majorTickMarkType, int minorTickMarkType) throws Exception
    {
        WorksheetCollection worksheets = workbook.getWorksheets();
        Worksheet worksheet = worksheets.get(0);
        //Set the name of worksheet
        worksheet.setName("3D Line");

        //Create chart
        ChartCollection charts = worksheet.getCharts();
        Chart chart = charts.get(charts.add(ChartType.LINE_3_D, 5, 1, 29, 10));

        //Set properties of nseries
        SeriesCollection nSeries = chart.getNSeries();
        nSeries.add("B2:F4", false);
        nSeries.setCategoryData("B1:F1");

        Cells cells = worksheet.getCells();
        String temp = "";
        for(int i = 0; i < nSeries.getCount(); i++)
        {
            //Set the name of nseries
            temp = cells.get(i + 1, 0).getStringValue();
            nSeries.get(i).setName(temp);
        }

        //Set properties of chart
        chart.setRotationAngle(30);

        //Set properties of chart title
        Title title = chart.getTitle();
        title.setText("Sales By Region");
        Font font1 = title.getTextFont();
        font1.setBold(true);
        font1.setColor(Color.getBlack());
        font1.setSize(12);

        //Set properties of categoryaxis title
        Axis categoryAxis = chart.getCategoryAxis();
        title = categoryAxis.getTitle();
        title.setText("Year(2002-2006)");
        Font font2 = title.getTextFont();
        font2.setBold(true);
        font2.setColor(Color.getBlack());
        font2.setSize(10);
        categoryAxis.getTickLabels().setRotationAngle(cLabelsRotation);

        //Set properies to valueaxis
        Axis valueAxis = chart.getValueAxis();
        valueAxis.getTickLabels().setRotationAngle(vLabelsRotation);
        switch (majorTickMarkType)
        {
            case 0:
                valueAxis.setMajorTickMark(TickMarkType.NONE);
                break;
            case 1:
                valueAxis.setMajorTickMark(TickMarkType.INSIDE);
                break;
            case 2:
                valueAxis.setMajorTickMark(TickMarkType.OUTSIDE);
                break;
            case 3:
                valueAxis.setMajorTickMark(TickMarkType.CROSS);
                break;
        }
        switch (minorTickMarkType)
        {
            case 0:
                valueAxis.setMinorTickMark(TickMarkType.NONE);
                break;
            case 1:
                valueAxis.setMinorTickMark(TickMarkType.INSIDE);
                break;
            case 2:
                valueAxis.setMinorTickMark(TickMarkType.OUTSIDE);
                break;
            case 3:
                valueAxis.setMinorTickMark(TickMarkType.CROSS);
                break;
        }

        //Set the position of legend
        Legend legend = chart.getLegend();
        legend.setPosition(LegendPositionType.TOP);
    }
}
